-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.21					---
--- Version: 0.4 [2012.02.09]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/autoconfirmroll
--- WoWInterface	http://www.wowinterface.com/downloads/info20516-AutoConfirmRoll.html

local NAME = ...
local VERSION = 0.4
local BUILD = "Release"

AutoConfirmRoll = LibStub("AceAddon-3.0"):NewAddon(NAME, "AceEvent-3.0", "AceConsole-3.0")
local ACR = AutoConfirmRoll

local ACR2 = LibStub("AceConfigRegistry-3.0")
local ACD = LibStub("AceConfigDialog-3.0")

local profile

	---------------
	--- Options ---
	---------------

local rollTypes = {}

local remap = {
	Need = 1,
	Greed = 2,
	Disenchant = 3,
}

local ITEM_QUALITY = {
	ITEM_QUALITY_COLORS[2].hex..ITEM_QUALITY2_DESC.."|r",
	ITEM_QUALITY_COLORS[3].hex..ITEM_QUALITY3_DESC.."|r",
	ITEM_QUALITY_COLORS[4].hex..ITEM_QUALITY4_DESC.."|r",
}

local options = {
	type = "group",
	name = NAME.." |cffADFF2Fv"..VERSION.."|r",
	handler = ACR,
	args = {
		inline1 = {
			type = "group", order = 1,
			name = " ",
			inline = true,
			get = "GetValue", set = "SetValueRoll",
			args = {
				Need = {
					type = "toggle", order = 1,
					width = "full", descStyle = "",
					name = "|TInterface\\Buttons\\UI-GroupLoot-Dice-Up:16:16|t "..NEED,
				},
				Greed = {
					type = "toggle", order = 2,
					width = "full", descStyle = "",
					name = "|TInterface\\Buttons\\UI-GroupLoot-Coin-Up:16:16|t "..GREED,
				},
				Disenchant = {
					type = "toggle", order = 3,
					width = "full", descStyle = "",
					name = "|TInterface\\Buttons\\UI-GroupLoot-DE-Up:16:16|t "..ROLL_DISENCHANT,
				},
			},
		},
		spacing = {type = "description", order = 2, name = " "},
		inline2 = {
			type = "group", order = 3,
			name = FILTERS,
			inline = true,
			get = "GetValue", set = "SetValueGroup",
			args = {
				Threshold = {
					type = "select", order = 1,
					descStyle = "",
					name = " "..LOOT_THRESHOLD,
					values = ITEM_QUALITY,
					get = function(i) return profile.Threshold end,
					set = function(i, v) profile.Threshold = v end,
				},
				spacing = {type = "description", order = 2, name = " "},
				Party = {
					type = "toggle", order = 3,
					width = "full", descStyle = "",
					name = " |cffA8A8FF"..PARTY.."|r",
				},
				Raid = {
					type = "toggle", order = 4,
					width = "full", descStyle = "",
					name = " |cffFF7F00"..RAID.."|r",
				},
			},
		},
	},
}

function ACR:GetValue(i)
	return profile[i[#i]]
end

function ACR:SetValueRoll(i, v)
	profile[i[#i]] = v
	rollTypes[remap[i[#i]]] = v
end

function ACR:SetValueGroup(i, v)
	profile[i[#i]] = v
	self:PARTY_MEMBERS_CHANGED(nil, true)
end

local defaults = {
	profile = {
		Need = true,
		Greed = true,
		Disenchant = true,
		Threshold = 3,
		Party = true,
		Raid = true,
	},
}

	---------------------------
	--- Ace3 Initialization ---
	---------------------------

function ACR:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AutoConfirmRollDB", defaults, true)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshDB")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshDB")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshDB")
	self:RefreshDB()
	
	self.db.global.version = VERSION
	self.db.global.build = BUILD
	
	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	ACR2:RegisterOptionsTable("AutoConfirmRoll_Main", options)
	ACR2:RegisterOptionsTable("AutoConfirmRoll_Profiles", profiles)
	
	ACD:AddToBlizOptions("AutoConfirmRoll_Main", NAME)
	ACD:AddToBlizOptions("AutoConfirmRoll_Profiles", profiles.name, NAME)
	
	self:RegisterChatCommand("acr", "SlashCommand")
	self:RegisterChatCommand("autoconfirmroll", "SlashCommand")
end

function ACR:OnEnable()
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("CONFIRM_LOOT_ROLL", "CONFIRM_ROLL")
	self:RegisterEvent("CONFIRM_DISENCHANT_ROLL", "CONFIRM_ROLL")
end

function ACR:RefreshDB()
	profile = self.db.profile

	rollTypes = {
		profile.Need,
		profile.Greed,
		profile.Disenchant,
	}
end

function ACR:SlashCommand(input)
	InterfaceOptionsFrame_OpenToCategory(NAME)
end

	--------------
	--- Events ---
	--------------

local cd = 0
local isParty, isRaid

function ACR:PARTY_MEMBERS_CHANGED(event, skip)
	if (time() > cd) or skip then
		cd = time() + .5
		local numRaidMembers = GetNumRaidMembers()
		isParty = profile.Party and (numRaidMembers == 0 and GetNumPartyMembers() > 0)
		isRaid = profile.Raid and numRaidMembers > 0
	end
end

function ACR:CONFIRM_ROLL(event, id, lootType)
	local qualityThreshold = select(4, GetLootRollItemInfo(id)) <= profile.Threshold + 1
	if rollTypes[lootType] and qualityThreshold and (isParty or isRaid) then
		ConfirmLootRoll(id, lootType)
	end
end

	---------------------
	--- LibDataBroker ---
	---------------------

local dataobject = {
	type = "launcher",
	icon = "Interface\\Buttons\\UI-GroupLoot-Dice-Up",
	text = NAME,
	OnClick = function(clickedframe, button)
		if InterfaceOptionsFrame:IsShown() then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(NAME)
		end
	end,
	OnTooltipShow = function(tt)
		tt:AddLine(NAME)
	end,
}

LibStub("LibDataBroker-1.1"):NewDataObject(NAME, dataobject)